/*
Demonstration of fitting a three-level hierarchical GLMM to incidence data, using
Laplacian approximation of the likelihood function, and the CLL link of the
expected probability of disease. Based on:
	Kriss, A. B., Paul, P. A., and Madden, L. V. 2012. Characterizing heterogeneity 
	of disease incidence in a spatial hierarchy: A case study from a decade of 
	observations of Fusarium head blight of wheat. Phytopathology 102: 867-877. 

Data are simulated here in a fashion that reflects the motivating example 
(Fusarium Head Blight (FHB) incidence in counties, fields within counties, 
and sites within fields within counties). 

This type of analysis (model fitting) can take a long time to run. Be patient!

*/

data a;
CALL STREAMINIT(12345);						*<--Initial seed for the random number generator (take 
												out to get different generated values--if you do not have
												this CALL, generated data will be different each time);
	inter = -2;								*<--Intercept of the equation on the CLL link scale (mu).
												mu=-2 translates to p = 1-exp(-exp(mu)) = 0.1265. ;
	n = 45;									*<--Number of observations per site;
	do county = 1 to 25;					*<--25 counties;
		varc = 0.65;						*<--Among-county variance in the simulation;
		uc = rand('normal')*sqrt(varc);		*<--random effect of county;
		do field = 1 to 5;					*<--five fields per county;
			varf = .50;						*<--Among-field (within county) variance in the simulation;
			uf = rand('normal')*sqrt(varf);	*<--random effect of field within county;
			do site = 1 to 10;				*<--10 sites per field per county
												(with 45 binary observations per site);
				vars = .07;					*<--Among-site (within county and field) variance in simulation;
				us = rand('normal')*sqrt(vars);	*<--random effect of site within field with county;
				
				eta = inter + uc + uf + us;	*<--eta (CLL(p)) as a function of mu (intercept), and the three 
												random-effect terms;
				p = (1-exp(-exp(eta)));		*<--Probability of disease--inverse link;
				y = rand('binomial',p,n);	*<--y is conditionally binomial;
				output;
				end;
			end;
		end;
run;
*proc print data=a; *run;

title 'Laplace ML model fitting for three-level hierarchy (with CLL link, conditional binomial)';
title2 'intercept=-2, variances: 0.65, .50, .07 [n=45], 25 counties, 5 fields/co, 10 sites/field';
proc glimmix data=a method=laplace ;
	class county field site;
	model y/n =  / dist=binomial link=cll  s ddfm=none;		*<--see eq. 3 in paper;
	random int field site(field) / subject=county   ;		*<--specify the three random effects.	
																('int' give county term here);	
	covtest  / cl(type=profile) ;							*<--get profile-likelihood confidence intervals;
	covtest 'county V=0?' 0 . .;							*<--tests of variance equalities to 0;
	covtest 'fi(co) V=0?' . 0 .;
	covtest 'si(fi co) V=0?' . . 0;
	covtest 'co V = fi V?' general 1 -1;					*<--test of equality of county and field var.;
	covtest 'fi V = si V?' general 0 1 -1;					*<--test of equality of field and site var.;
	covtest '(co&fi/2 V = si V?' general 1 1 -2;			*--(not in paper): equality of field and county 
																to site variance;
	estimate 'mu (and p)' int 1 / cl ilink;
	estimate 'EBLUP co 1' int 1 |  int 1 / subject 1       	cl ilink;	*<--some example EBLUPs;
	estimate 'EBLUP co 2' int 1 |  int 1 / subject 0 1    	cl ilink;
	estimate 'EBLUP co 4' int 1 |  int 1 / subject 0 0 0 1 	cl ilink;
	estimate 'EBLUP co 2, field 2' int 1 | int 1 field 0 1     / subject 0 1      cl ilink;
	estimate 'EBLUP co 4, field 4' int 1 | int 1 field 0 0 0 1 / subject 0 0 0 1  cl ilink;
run;
title; title2;
*---Note: Mean toward the end of the row in Estimates output table is the inverse link (last four columns).
	This is the estimated probability of disease (p) and related statistics.;
*---There are multiple ways of writing the RANDOM statement(s), but this one generally
	requires the least amount of memory, and is the best choice for obtaining the 
	tests of variances or the confidence intervals for variances.;
*---If you want the solution for the random effects (i.e., the effects of each individual
	county, field(county) or site(county field), include an "s" as an option on the 
	random statement. This will give you a lot of output.;
run;

